within AICE.Fluid.Fittings.Orifices;

model FixedOrifice "Variable Orifice Based on PERRY's equations"

  // interfaces
  extends Modelica.Fluid.Interfaces.PartialTwoPortTransport(
    showDesignFlowDirection = false
  );

  parameter SI.Diameter diameter(displayUnit="mm") = 0.01 "Valve Diameter";

  //*****************************  DECLARATIONS **********************************

protected

  parameter SI.Area A_actual = Modelica.Constants.pi * (diameter/2)^2 "Actual area of valve";
  SI.Area A_effective "Effective area of valve";

  Real ratio_p;
  Real ratio_criteria;
  Real ratio;
  Real gamma;

  Real areaCOEF;
  Real mFlowCOEF;
  Real direction;

  Medium.SpecificHeatCapacity R_upwind "Upwind gas constant";
  Medium.SpecificHeatCapacity Cv_upwind "Upwind Cv";
  Medium.SpecificHeatCapacity Cp_upwind "Upwind Cp";

  Medium.Temperature T_upwind "Upwind temperature";

  SI.AbsolutePressure p_upwind "Upwind pressure";
  SI.AbsolutePressure p_downwind "Downwind pressure";

equation

  R_upwind = Utilities.regStep(
    dp,
    Medium.gasConstant(state_a), Medium.gasConstant(state_b)
  );

  Cv_upwind = Utilities.regStep(
    dp,
    Medium.specificHeatCapacityCv(state_a), Medium.specificHeatCapacityCv(state_b)
  );

  Cp_upwind = Utilities.regStep(
    dp,
    Medium.specificHeatCapacityCp(state_a), Medium.specificHeatCapacityCp(state_b)
  );

  T_upwind = Utilities.regStep(
    dp,
    port_a_T, port_b_T
  );
  
  direction = Utilities.regStep(
    dp,
    1, -1
  );

  p_upwind   = max(port_a.p, port_b.p);
  p_downwind = min(port_a.p, port_b.p);

  ratio_p = p_downwind / p_upwind;
  gamma = Cp_upwind / Cv_upwind;

  ratio_criteria = (2 / (gamma + 1)) ^ (gamma / (gamma - 1));
  ratio = max(ratio_p, ratio_criteria);

  mFlowCOEF = sqrt(2 * gamma / (gamma - 1)) * sqrt(abs(ratio ^ (2 / gamma) - ratio ^ ((gamma + 1) / gamma)));
  areaCOEF = 0.8414 + ratio_p * (ratio_p * (0.8415 + ratio_p * (ratio_p * (4.6 - ratio_p * 1.6827) - 3.9)) - 0.1);

  A_effective = A_actual * areaCOEF;

  m_flow = homotopy(
    direction * (A_effective * mFlowCOEF * p_upwind / sqrt(R_upwind * T_upwind)),
    direction * (A_actual * mFlowCOEF * p_upwind / sqrt(R_upwind * T_upwind))
  );

  // Isenthalpic state transformation (no storage and no loss of energy)
  port_a.h_outflow = inStream(port_b.h_outflow);
  port_b.h_outflow = inStream(port_a.h_outflow);

  annotation(
    Icon(graphics = {Line(origin = {2, -40}, points = {{-42, 0}, {38, 0}}, thickness = 0.75), Line(origin = {0, 23.77}, points = {{0, -16.235}, {-20, 15.765}, {20, 15.765}, {0, -16.235}}, thickness = 0.75), Line(origin = {0, -24.24}, points = {{0, 16.235}, {-20, -15.765}, {20, -15.765}, {0, 16.235}}, thickness = 0.75), Line(origin = {0, 8}, points = {{-40, 32}, {40, 32}}, thickness = 0.75), Line(points = {{-80, 0}, {80, 0}})}, coordinateSystem(initialScale = 0.1)));
end FixedOrifice;
