within AICE.Fluid.Pipes.BaseClasses;

model PartialTimeBasedReaction
  replaceable package Medium = AICE.Media.Interfaces.PartialCombustionMixtureMedium;
  
  extends PartialPipeReactionModel;
  constant Integer nReaction = 1;

  SI.Time t_reaction = time;
  constant SI.StoichiometricNumber[nReaction, Medium.nXi] stoichiometric_reactants = fill(1 / Medium.nXi, nReaction, Medium.nXi), stoichiometric_products = fill(1 / Medium.nXi, nReaction, Medium.nXi);

  parameter SI.AmountOfSubstance reactableAmount_eps = 1e-9;
  output SI.MolarFlowRate[n, nReaction] reactionRates;
  output SI.HeatFlowRate[n, nReaction] reactionHeatFlows;
  SI.AmountOfSubstance[n, nReaction] reactableAmounts;
  // Trigger booleans
  Boolean[n, nReaction] reacts(each start = false);
  Boolean reaction_trigger = true;

  discrete SI.Time[n, nReaction] t_previous;
  SI.Time[n, nReaction] t_step = fill(time, n, nReaction) - t_previous[:, :];

  // Thermodynamic factors
  constant SI.MassFraction[Medium.nXi, Medium.nXi] mXis_identity = identity(Medium.nXi);
  SI.AbsolutePressure[n] ps = Medium.pressure(states);
  SI.Temperature[n] Ts = Medium.temperature(states);
  SI.MolarInternalEnergy[n, Medium.nXi] Hs_Xi;
  SI.MolarInternalEnergy[n, nReaction] Hs_products, Hs_reactants, Hs_balance;
initial equation
  pre(reaction_trigger) = true;
  pre(reacts[:, :]) = fill(false, n, nReaction);
equation
/**
   * Original output variables of `PartialChamberReactionModel`
   * - The molar reaction rate `MbXi_flow` of each substance
   * - The total reaction heat `Qb_flow`
   * are calculated from the reaction rate and heat for each reaction number.
   *
   * Complete model must implement `reaction*` variables showned below.
   * The variables are defined on each segment (1:n) and for each reaction NO (1:nReaction).
   * - `reactionRates[n, nReaction] (unit="mol/s")` reaction rate
   * - `reactionHeatFlows[n, Reaction] (unit="W")`  reaction heat
   */
  for i_sv in 1:n loop
    for iXi in 1:Medium.nXi loop
      MbXi_reacts[iXi, i_sv] = sum((stoichiometric_products[jReaction, iXi] - stoichiometric_reactants[jReaction, iXi]) * reactionRates[i, jReaction] for i in 1:n, jReaction in 1:nReaction);
    end for;
  end for;
  //Qb_flow = sum(reactionHeatFlows);
/**
   * Remaining reactable amount of substance for `reactionRate` calcuration.
   * It is always 0 or more.
   */
  for i in 1:n loop
    for jReaction in 1:nReaction loop
      reactableAmounts[i, jReaction] =   
       AICE.PowerUnits.Combustions.Chambers.BaseClasses.ReactionModels.Functions.calcurateReactableAmount(
       stoichiometric = stoichiometric_reactants[jReaction, :], MXi = MXis[i, :]
       );
    end for;
  end for;
  
  for i in 1:n loop
    for jReaction in 1:nReaction loop
/**
       * `reacts` :: reaction judgement
       * If {reactable amount is close to zero}              : stop     reaction (reacts = false)
       * Else if {at the moment trigger becomes true}        : start    reaction (reacts = true )
       * Else if {reaction has occured in the previous step} : continue reaction (reacts = true )
       * Else                                                : no       reaction (reacts = false)
       */
      reacts[i, jReaction] = reactableAmounts[i, jReaction] > reactableAmount_eps and (edge(reaction_trigger) or pre(reacts[i, jReaction]));
/**
       * `phi_currentReaction` :: phi_piston at reaction begins (discrete)
       * If {at the moment reaction start} : update value
       * Else                              : hold   value
       */
      t_previous[i, jReaction] = if edge(reacts[i, jReaction]) then t_reaction else pre(t_previous[i, jReaction]);
    end for;
  end for;
// # Thermodynamic factors
// ## Molar specific enthalpies
  for i in 1:n loop
    for jXi in 1:Medium.nXi loop
      Hs_Xi[i, jXi] = Medium.specificEnthalpy_pTX(ps[i], Ts[i], mXis_identity[jXi, :]) * Medium.MMX[jXi];
    end for;
    for jReaction in 1:nReaction loop
      Hs_products[i, jReaction] = sum(Hs_Xi[i, :] .* stoichiometric_products[jReaction, :]);
      Hs_reactants[i, jReaction] = sum(Hs_Xi[i, :] .* stoichiometric_reactants[jReaction, :]);
    end for;
  end for;
  Hs_balance = Hs_reactants .- Hs_products;
  
  //工事中　化学反応によるMass
  for i in 1:n loop
    for jXi in 1:Medium.nXi loop
      MbXi_reacts[i, jXi] = sum((stoichiometric_products[jReaction, jXi] - stoichiometric_reactants[jReaction, jXi]) * reactionRates[i, jReaction] for i in 1:n, jReaction in 1:nReaction);
    end for;
    for jReaction in 1:nReaction loop
      Hs_products[i, jReaction] = sum(Hs_Xi[i, :] .* stoichiometric_products[jReaction, :]);
      Hs_reactants[i, jReaction] = sum(Hs_Xi[i, :] .* stoichiometric_reactants[jReaction, :]);
    end for;
  end for;
  Hs_balance = Hs_reactants .- Hs_products;
  
end PartialTimeBasedReaction;