within AICE.Fluid.Sources;

model AngleBasedInjector

  import AICE.Math.AngleDifference4Stroke;

  import Modelica.Media.Interfaces.Choices.IndependentVariables;
  extends Modelica.Fluid.Sources.BaseClasses.PartialFlowSource;
  extends AICE.Icons.Fluid.Injector;

  Modelica.Blocks.Interfaces.RealInput phi_piston(final unit="rad")
  annotation(
    Placement(
      visible = true,
      transformation(origin = {-100, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 0),
      iconTransformation(origin = {-100, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 0)
    )
  );

  parameter SI.Mass fuelPerCycle(displayUnit = "g") = 0.02e-3;
  parameter SI.Angle injectionStart(displayUnit = "deg") = 0 "BTDC from compression";
  parameter SI.Angle injectionPeriod(displayUnit = "deg") = CarUnits.Conversions.from_deg(1);

  SI.Mass fuelRemained;
  Boolean injects;

  parameter SI.AbsolutePressure reference_p_vap = 1.e5
    "Reference pressure of vaporization";
  parameter SI.Temperature reference_T_vap = 293.15
    "Reference temperature of vaporization";
  parameter SI.SpecificEnthalpy reference_h_vap = 0.0
    "Reference specific enthalpy of vaporization";

  parameter SI.SpecificHeatCapacity Cp = 2000.0;
  parameter SI.Temperature reference_T_inj = 373.15
    "Reference temperature of Liquid Fuel in Rail";

  Medium.ThermodynamicState state_vap
    "Thermodynamics state of vaporized gas";

protected

  SI.MassFlowRate fuelFlowRate_internal
    "Calcurated fuel flow rate";
  SI.SpecificEnthalpy h_internal
    "Calcurated Specific Enthalpy";

initial equation
  fuelRemained = 0.0;

equation

  fuelFlowRate_internal + der(fuelRemained) = 0;

  when ( AngleDifference4Stroke.compare(phi_piston, injectionStart) > 0 ) then
    reinit(
      fuelRemained,
      pre(fuelRemained) + fuelPerCycle
    );
  end when;

  injects = (0 <= fuelRemained);
  if injects then
    fuelFlowRate_internal = fuelPerCycle / injectionPeriod * der(phi_piston);
  else
    fuelFlowRate_internal = 0;
  end if;

  state_vap = Medium.setState_pTX(
    reference_p_vap,
    reference_T_vap,
    Medium.reference_X_fuel
  );

  h_internal = (
    Medium.specificEnthalpy(state_vap)
    - reference_h_vap
    + Cp * (reference_T_inj - reference_T_vap)
  );

  Modelica.Fluid.Utilities.checkBoundary(Medium.mediumName, Medium.substanceNames,
    Medium.singleState, true, Medium.reference_X_fuel, "AngleBasedInjector");

  if Medium.ThermoStates == IndependentVariables.ph or
     Medium.ThermoStates == IndependentVariables.phX then
     medium.h = h_internal;
  else
     medium.T = Medium.temperature(Medium.setState_phX(medium.p, h_internal, Medium.reference_X_fuel));
  end if;
  sum(ports.m_flow) = -fuelFlowRate_internal;
  medium.Xi = Medium.reference_X_fuel[1:Medium.nXi];
  ports.C_outflow = fill(0.0, nPorts, Medium.nC);

end AngleBasedInjector;
