within AICE.Fluid.Valves;

model ButterflyValve "Variable Orifice adapted to butterfly valve"

  // interfaces
  extends Modelica.Fluid.Interfaces.PartialTwoPortTransport(
    showDesignFlowDirection = false
  );

  Modelica.Blocks.Interfaces.RealInput phi_opening(
      final unit="rad"
    ) "Opening Angle. 0 if fully closed, π/2 if fully open." annotation(
    Placement(
      visible = true,
      transformation(origin = {0, 100}, extent = {{20, -20}, {-20, 20}}, rotation = 90),
      iconTransformation(origin = {0, 80}, extent = {{20, -20}, {-20, 20}}, rotation = 90)
    )
  );

  parameter SI.Diameter diameter(displayUnit="mm") = 0.1 "Valve Diameter";

protected

  parameter SI.Area A_actual = Modelica.Constants.pi * (diameter/2)^2 "Actual area of valve";
  SI.Area A_effective "Effective area of valve";

  Real ratio_p;
  Real ratio_criteria;
  Real ratio;
  Real gamma "heat capacity ratio";

  Real areaCOEF;
  Real mFlowCOEF;
  Real direction;

  Medium.SpecificHeatCapacity R_upwind "Upwind gas constant";
  Medium.SpecificHeatCapacity Cv_upwind "Upwind Cv";
  Medium.SpecificHeatCapacity Cp_upwind "Upwind Cp";

  Medium.Temperature T_upwind "Upwind temperature";

  SI.AbsolutePressure p_upwind "Upwind pressure";
  SI.AbsolutePressure p_downwind "Downwind pressure";

equation

  R_upwind = Utilities.regStep(
    dp,
    Medium.gasConstant(state_a), Medium.gasConstant(state_b)
  );

  Cv_upwind = Utilities.regStep(
    dp,
    Medium.specificHeatCapacityCv(state_a), Medium.specificHeatCapacityCv(state_b)
  );

  Cp_upwind = Utilities.regStep(
    dp,
    Medium.specificHeatCapacityCp(state_a), Medium.specificHeatCapacityCp(state_b)
  );

  T_upwind = Utilities.regStep(
    dp,
    port_a_T, port_b_T
  );
  
  direction = Utilities.regStep(
    dp,
    1, -1
  );

  p_upwind   = max(port_a.p, port_b.p);
  p_downwind = min(port_a.p, port_b.p);

  ratio_p = p_downwind / p_upwind;
  gamma = Cp_upwind / Cv_upwind;

  ratio_criteria = (2 / (gamma + 1)) ^ (gamma / (gamma - 1));
  ratio = max(ratio_p, ratio_criteria);

  mFlowCOEF = sqrt(2 * gamma / (gamma - 1)) * sqrt(ratio ^ (2 / gamma) - ratio ^ ((gamma + 1) / gamma));
  areaCOEF = 0.8414 + ratio_p * (ratio_p * (0.8415 + ratio_p * (ratio_p * (4.6 - ratio_p * 1.6827) - 3.9)) - 0.1);

  A_effective = A_actual * areaCOEF * (1.0 - abs(cos(phi_opening)));

  m_flow = homotopy(
    direction * (A_effective * mFlowCOEF * p_upwind / sqrt(R_upwind * T_upwind)),
    direction * (A_actual * mFlowCOEF * p_upwind / sqrt(R_upwind * T_upwind))
  );

  // Isenthalpic state transformation (no storage and no loss of energy)
  port_a.h_outflow = inStream(port_b.h_outflow);
  port_b.h_outflow = inStream(port_a.h_outflow);

  annotation(
    Icon(graphics = {Line(origin = {2.50481, -59.6875}, points = {{-42, 0}, {38, 0}}), Line(origin = {-0.504808, 28.1923}, points = {{-40, 32}, {40, 32}}), Line(points = {{-80, 0}, {80, 0}}), Rectangle(rotation = -30,fillColor = {115, 210, 22}, fillPattern = FillPattern.Solid, extent = {{-6, 58}, {6, -58}}), Ellipse(origin = {0, -1}, fillColor = {115, 210, 22}, fillPattern = FillPattern.Solid, extent = {{-14, 15}, {14, -13}}, endAngle = 360), Line(origin = {-30, -70}, points = {{-10, -10}, {10, 10}, {10, 10}}), Line(origin = {-9.90385, -70.0962}, points = {{-10, -10}, {10, 10}, {10, 10}}), Line(origin = {10.1923, -70.6972}, points = {{-10, -10}, {10, 10}, {10, 10}}), Line(origin = {29.2788, -69.7837}, points = {{-10, -10}, {10, 10}, {10, 10}}), Line(origin = {-10.5048, 70.649}, points = {{-10, -10}, {10, 10}, {10, 10}}), Line(origin = {-30.601, 70.7452}, points = {{-10, -10}, {10, 10}, {10, 10}}), Line(origin = {9.59134, 70.048}, points = {{-10, -10}, {10, 10}, {10, 10}}), Line(origin = {28.6778, 70.9615}, points = {{-10, -10}, {10, 10}, {10, 10}})}, coordinateSystem(initialScale = 0.1)));
end ButterflyValve;
