within AICE.Fluid.Valves;

model CamBasedValve "Valve with cam angle based tables of lift and discharge coefficients with controllable offset"

  // interfaces
  extends Modelica.Fluid.Interfaces.PartialTwoPortTransport(
    showDesignFlowDirection = false
  );

  Modelica.Blocks.Interfaces.RealInput phi_crank annotation(
    Placement(
      visible = true,
      transformation(origin = {0, 100}, extent = {{-20, -20}, {20, 20}}, rotation = -90),
      iconTransformation(origin = {0, 88}, extent = {{-12, -12}, {12, 12}}, rotation = -90)
    )
  );

  Modelica.Blocks.Interfaces.RealInput phi_offset annotation(
    Placement(
      visible = true,
      transformation(origin = {-100, 60}, extent = {{-20, -20}, {20, 20}}, rotation = 0),
      iconTransformation(origin = {-87, 59}, extent = {{-13, -13}, {13, 13}}, rotation = 0)
    )
  );

  Modelica.Blocks.Interfaces.BooleanOutput opens
    "= true, valve opens" annotation(
    Placement(
      visible = true,
      transformation(origin = {100, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0),
      iconTransformation(origin = {100, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  //*********** GEOMETRY ***************
  parameter SI.Diameter diameter(displayUnit="mm") = 0.01 "Valve Diameter";
  parameter Real phi_reference(displayUnit="deg") = 0.0 "deg Reference Angle";
  parameter Real camTableStartAngle(displayUnit="deg") = 0.0 "deg Lobe Center Reference Angle";

  SI.Angle crankAngle;

//  Real TH_DEG(displayUnit="deg");
  Real phi_lift(displayUnit="deg");
  parameter Real phi_deadZone(displayUnit="deg") = 170.0 "deg Dead Zone ";

  // ********** TABLES **************
  parameter String fileName_lift = Files.loadResource("modelica://AICE.ENG.VAL/DAT/valve_lift.txt") "input file name of lift function of angle : LIFT=FCT(ANGLE)" annotation(
    Dialog(loadSelector(filter = "Text files (*.txt);;DAT DAT-files (*.dat)", caption = "Open file in which table is present")));
  parameter String fileName_cd = Files.loadResource("modelica://AICE.ENG.VAL/DAT/valve_cd.txt") "input file name of dischage function of lift : CD=FCT(LIFT)" annotation(
    Dialog(loadSelector(filter = "Text files (*.txt);;DAT DAT-files (*.dat)", caption = "Open file in which table is present")));
  Modelica.Blocks.Tables.CombiTable1D angle2lift(fileName = fileName_lift, tableName = "lift", tableOnFile = true) annotation(
    Placement(visible = true, transformation(origin = {10, -68}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Tables.CombiTable1D lift2cd(fileName = fileName_cd, tableName = "cd", tableOnFile = true) annotation(
    Placement(visible = true, transformation(origin = {46, -68}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));

protected

  parameter SI.Area A_actual = Modelica.Constants.pi * (diameter/2)^2 "Actual area of valve";
  SI.Area A_effective "Effective area of valve";

  Real ratio_p;
  Real ratio_criteria;
  Real ratio;
  Real gamma "heat capacity ratio";

  Real areaCOEF;
  Real mFlowCOEF;
  Real direction;

  Medium.SpecificHeatCapacity R_upwind "Upwind gas constant";
  Medium.SpecificHeatCapacity Cv_upwind "Upwind Cv";
  Medium.SpecificHeatCapacity Cp_upwind "Upwind Cp";

  Medium.Temperature T_upwind "Upwind temperature";

  SI.AbsolutePressure p_upwind "Upwind pressure";
  SI.AbsolutePressure p_downwind "Downwind pressure";

  parameter SI.Angle phi_2cycle(displayUnit = "rad") = 4 * Modelica.Constants.pi;
  parameter SI.Angle referenceAngle(displayUnit="rad") = phi_reference / 180.0 * Modelica.Constants.pi "Reference Angle";  
  parameter SI.Angle deadZoneAngle(displayUnit="rad") = phi_deadZone / 180.0 * Modelica.Constants.pi "Dead Zone";

equation

  R_upwind = Utilities.regStep(
    dp,
    Medium.gasConstant(state_a), Medium.gasConstant(state_b)
  );

  Cv_upwind = Utilities.regStep(
    dp,
    Medium.specificHeatCapacityCv(state_a), Medium.specificHeatCapacityCv(state_b)
  );

  Cp_upwind = Utilities.regStep(
    dp,
    Medium.specificHeatCapacityCp(state_a), Medium.specificHeatCapacityCp(state_b)
  );

  T_upwind = Utilities.regStep(
    dp,
    port_a_T, port_b_T
  );
  
  direction = Utilities.regStep(
    dp,
    1, -1
  );

  p_upwind   = max(port_a.p, port_b.p);
  p_downwind = min(port_a.p, port_b.p);

  ratio_p = p_downwind / p_upwind;
  gamma = Cp_upwind / Cv_upwind;

  ratio_criteria = (2 / (gamma + 1)) ^ (gamma / (gamma - 1));
  ratio = max(ratio_p, ratio_criteria);

 // phi_crank is a Real we consider an angle in RADians out of a sensor on the Crankshaft
 // CENTER_DEF is the position of the Lob center angle position relatively to 
 //    the crank shaft angle TDC 
 // ************* INPUT ANGLE ************
  crankAngle = mod(phi_crank - referenceAngle , phi_2cycle);
//  TH_DEG = phi_crank * 180/Modelica.Constants.pi;
  phi_lift = crankAngle * 180/Modelica.Constants.pi - camTableStartAngle - 360.0;

  angle2lift.u[1] = if phi_crank < deadZoneAngle
                      then 0.0 
                      else phi_lift; // TODO: homotopy
  
  opens = (angle2lift.y[1] > 1.e-5);
  
  connect(angle2lift.y[1], lift2cd.u[1]) annotation(
    Line(points = {{21, -68}, {34, -68}}, color = {0, 0, 127}, thickness = 0.5));


  mFlowCOEF = sqrt(2 * gamma / (gamma - 1)) * sqrt(ratio ^ (2 / gamma) - ratio ^ ((gamma + 1) / gamma));
  areaCOEF = lift2cd.y[1];

  A_effective = areaCOEF * A_actual;

  m_flow = homotopy(
    direction * (A_effective * mFlowCOEF * p_upwind / sqrt(R_upwind * T_upwind)),
    direction * (A_actual * mFlowCOEF * p_upwind / sqrt(R_upwind * T_upwind))
  );

  // Isenthalpic state transformation (no storage and no loss of energy)
  port_a.h_outflow = inStream(port_b.h_outflow);
  port_b.h_outflow = inStream(port_a.h_outflow);

  annotation(
    Icon(graphics = {Text(origin = {41, 81}, extent = {{-19, 19}, {19, -19}}, textString = "CrankAngle"), Text(origin = {-81, 83}, extent = {{-19, 19}, {19, -19}}, textString = "OffsetAngle"), Text(origin = {-42, 30}, extent = {{-26, 58}, {14, 22}}, textString = "-"), Text(origin = {-16, 42}, extent = {{-26, 58}, {14, 22}}, textString = "+"), Line(origin = {-35.4684, 74.6203}, points = {{-12, 16}, {12, -16}}, thickness = 1.25), Ellipse(origin = {-35, 73}, lineColor = {78, 154, 6}, lineThickness = 1, extent = {{-27, 23}, {27, -23}}, endAngle = 360), Line(origin = {-58.9747, 59.33}, points = {{-23, 0}, {-1, 0}}, color = {78, 154, 6}, pattern = LinePattern.Dash, thickness = 1.25, arrow = {Arrow.None, Arrow.Filled}, arrowSize = 8), Line(origin = {-0.63, 0}, points = {{-50, 0}, {50, 0}, {50, 0}}, color = {78, 154, 6}, thickness = 1.5), Line(origin = {-1.02, -8.81}, points = {{-50.9774, 8.80662}, {-38.9774, -9.19338}, {43.0226, -9.19338}, {51.0226, 6.80662}}, color = {78, 154, 6}, thickness = 1.5), Line(origin = {-31.14, 27.86}, points = {{-20.8592, -27.8592}, {1.14077, -19.8592}, {17.1408, -1.85923}, {21.1408, 8.14077}, {21.1408, 28.1408}, {21.1408, 28.1408}}, color = {78, 154, 6}, thickness = 1.5), Line(origin = {-31.14, 27.86}, points = {{79.1408, -27.8592}, {63.1408, -19.8592}, {43.1408, -1.85923}, {41.1408, 6.14077}, {41.1408, 30.1408}, {21.1408, 28.1408}}, color = {78, 154, 6}, thickness = 1.5), Line(origin = {1.01, -19.9}, points = {{-41.0082, 1.90344}, {-21.0082, -2.09656}, {-1.00821, -2.09656}, {22.9918, -2.09656}, {40.9918, 1.90344}}, color = {78, 154, 6}, thickness = 1.5), Line(origin = {-79, 0}, points = {{-21, 0}, {21, 0}}, thickness = 1.5), Line(origin = {81, 0}, points = {{-21, 0}, {21, 0}}, thickness = 1.5)}, coordinateSystem(initialScale = 0.1)));
end CamBasedValve;
