within AICE.PowerUnits.Combustions.Chambers.BaseClasses.HeatTransfers;

model WoschniHeatTransfer

  import Modelica.Constants.pi;

  extends PartialChamberHeatTransfer;

  Modelica.Blocks.Interfaces.RealInput phi_piston(final unit="rad")
  annotation(
    Placement(
      visible = true,
      transformation(origin = {-70, 40}, extent = {{-5, -5}, {5, 5}}, rotation = -90)
    )
  );

  Modelica.Blocks.Interfaces.BooleanInput combusts
  annotation(
    Placement(
      visible = true,
      transformation(origin = {-50, 40}, extent = {{-5, -5}, {5, 5}}, rotation = -90)
    )
  );

  Modelica.Blocks.Interfaces.BooleanInput intakeValveOpens
  annotation(
    Placement(
      visible = true,
      transformation(origin = {-30, 40}, extent = {{-5, -5}, {5, 5}}, rotation = -90)
    )
  );

  Modelica.Blocks.Interfaces.BooleanInput exhaustValveOpens
  annotation(
    Placement(
      visible = true,
      transformation(origin = {-10, 40}, extent = {{-5, -5}, {5, 5}}, rotation = -90)
    )
  );

  // geometry parameters
  parameter SI.Diameter bore;
  parameter SI.Length pistonStroke;
  parameter SI.Volume V_BDC;

  // thermodynamic parameter(s)
  parameter SI.RatioOfSpecificHeatCapacities gamma = 1.34;

  // general thermodynamics variables
  Medium.AbsolutePressure[n] ps         = Medium.pressure(states);
  //SI.Volume[n]               Vs         = ms[:] ./ Medium.density(states[:]);
    SI.Density[n] Vd(each min=0.01, each max=100) = Medium.density(states[:]);
    SI.Volume               Vs[n](each min=10e-5, each max=0.001)         = {ms[i] / max(0.01,Vd[i]) for i in 1:n};
    //SI.Volume               Vs[n]         = {ms[i] / max(0.01,Medium.density(states[i])) for i in 1:size(states,1)} ;
    //SI.Volume               Vs[n]         = {ms[i] / max(0.01,ps[i]) for i in 1:n} ;

  SI.Mass[n]                 ms;
  SI.Temperature[n]          Ts_chamber = Ts;
  SI.Temperature[n]          Ts_wall    = heatPorts[:].T;

  // Woschni model variables
  SI.CoefficientOfHeatTransfer[n] ks_woschni;
  SI.Velocity[n] us;

  // Woschni C1 effect (piston)
  type C1 = Real(unit = "1") "Ratio of chamber flow velocity to piston average speed";
  parameter C1 c1_intakeExhaust    = 6.18 "C1 at intake, exhaust stroke";
  parameter C1 c1_compressionPower = 2.28 "C1 at compression, power stroke";
  C1 c1;
  SI.Velocity u_piston = c1 * v_piston_average;
  SI.Velocity v_piston_average;

  // Woschni C2 effect (combustion)
  type C2 = Real;
  parameter C2 c2 = 3.24e-3;
  SI.Velocity[n]             us_combustion;
  Boolean                    combustionAffectsHeatTransfer(start=false);
  Medium.AbsolutePressure[n] ps_motoring;
  Medium.AbsolutePressure[n] ps_combustionBegin;
  SI.Volume[n]               Vs_combustionBegin;
  CarUnits.CompressionRatio  compression;

equation

  Q_flows[:] = ks_woschni[:] .* surfaceAreas[:] .* (Ts_wall[:] .- Ts_chamber[:]);

  for i in 1:n loop
    ks_woschni[i] = 0.013 * bore^(-0.2) * ps[i]^0.8 * Ts_chamber[i]^(-0.53) * us[i]^0.8;

  end for;

  // flow velocity assumption affected by piston and combustion
  us[:] = fill(u_piston, n) .+ us_combustion[:];

  // effect by piston
  v_piston_average / pistonStroke = der(phi_piston) / pi "Half a cycle per stroke";

  if intakeValveOpens or exhaustValveOpens then // any valve opens
    c1 = c1_intakeExhaust;
  else
    c1 = c1_compressionPower;
  end if;

  // effect by combustion
  combustionAffectsHeatTransfer = (
    edge(combusts)
    or (
      pre(combustionAffectsHeatTransfer) and
      not edge(exhaustValveOpens)
    )
  );

  der(ps_combustionBegin[:]) = zeros(n);
  der(Vs_combustionBegin[:]) = zeros(n);

  when edge(combusts) then
    for i in 1:n loop
      reinit(ps_combustionBegin[i], ps[i]);
      reinit(Vs_combustionBegin[i], Vs[i]);
    end for;
  end when;

  if (not combustionAffectsHeatTransfer) then
    us_combustion[:] = fill(0.0, n);
    ps_motoring[:]   = ps[:];
  else
    for i in 1:n loop
      us_combustion[i] = (
        c2
        * max(ps[i]-ps_motoring[i], 0.0) / ps[i]
        * compression
        * Ts_chamber[i]
      );

      ps_motoring[i] * Vs[i] ^ gamma
        = ps_combustionBegin[i] * Vs_combustionBegin[i] ^ gamma;
    end for;
  end if;

  // trivial equations
  for i in 1:n loop
    ms[i] = sum(mXis[i,:]);
  end for;

  compression = V_BDC / max(1e-5,sum(Vs)) ;

end WoschniHeatTransfer;
