within AICE.PowerUnits.Combustions.Chambers.BaseClasses;

model PartialLumpedChamber_b "Lumped volume with a vector of fluid ports, replaceable heat transfer model and replaceable reaction model"
  extends Modelica.Fluid.Interfaces.PartialLumpedVolume(redeclare replaceable package Medium = Media.Interfaces.PartialCombustionMixtureMedium, final use_T_start = false);


  parameter Medium.MassFlowRate m_flow_nominal = if system.use_eps_Re then system.m_flow_nominal else 1e2 * system.m_flow_small "Nominal value for mass flow rates in ports" annotation(
    Dialog(tab = "Advanced", group = "Port properties"));
  parameter SI.MassFlowRate m_flow_small(min = 0) = if system.use_eps_Re then system.eps_m_flow * m_flow_nominal else system.m_flow_small "Regularization range at zero mass flow rate" annotation(
    Dialog(tab = "Advanced", group = "Port properties"));
  parameter Boolean use_Re = system.use_eps_Re "= true, if turbulent region is defined by Re, otherwise by m_flow_small" annotation(
    Dialog(tab = "Advanced", group = "Port properties"),
    Evaluate = true);
  SI.HeatFlowRate Qb_comb;

  // Heat transfer through boundary
  parameter Boolean use_HeatTransfer = false "= true to use the HeatTransfer model" annotation(
    Dialog(tab = "Assumptions", group = "Heat transfer"));
  replaceable model HeatTransfer = AICE.PowerUnits.Combustions.Chambers.BaseClasses.HeatTransfers.IdealHeatTransfer constrainedby AICE.PowerUnits.Combustions.Chambers.BaseClasses.HeatTransfers.PartialChamberHeatTransfer "Wall heat transfer" annotation(
     Dialog(tab = "Assumptions", group = "Heat transfer", enable = use_HeatTransfer),
     choicesAllMatching = true);
  HeatTransfer heatTransfer(redeclare final package Medium = Medium, final n = 1, final states = {medium.state}, final mXis = {mXi}, final use_k = use_HeatTransfer) annotation(
    Placement(visible = true, transformation(origin = {-60, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 90)));
  Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort if use_HeatTransfer annotation(
    Placement(transformation(extent = {{-110, -10}, {-90, 10}})));
  SI.MassFlowRate mb_flow_react;
  SI.MassFlowRate[Medium.nXi] mbXi_flow_p;
  SI.AmountOfSubstance[Medium.nXi] MXi "Moles of independent components in the fluid";


  // Note: should use fluidLevel_start - portsData.height
  constant SI.ReynoldsNumber Re_turbulent = 100 "cf. suddenExpansion";

protected
  input SI.Height fluidLevel = 0 "level of fluid in the vessel for treating heights of ports";
  parameter SI.Height fluidLevel_max = 1 "maximum level of fluid in the vessel";
  parameter SI.Area vesselArea = Modelica.Constants.inf "Area of the vessel used to relate to cross flow area of ports";

equation

// mbXi_flow_pの算出はu側で計算したものをバインドする前提
  MXi[:] .* Medium.MMX[:] = mXi[:];
  mb_flow = mb_flow_react;
// apply reaction mass consumption
  mbXi_flow = mbXi_flow_p[:];
// apply reactionModel reactants only
  //mbC_flow = {0.0};
// Hb_flowはWiebeWoscniChamber2zonesBに実装
  Qb_flow = heatTransfer.Q_flows[1] + Qb_comb;

// Check for correct solution
  assert(fluidLevel <= fluidLevel_max, "Vessel is overflowing (fluidLevel > fluidLevel_max = " + String(fluidLevel) + ")");
  assert(fluidLevel > (-1e-6 * fluidLevel_max), "Fluid level (= " + String(fluidLevel) + ") is below zero meaning that the solution failed.");
// Boundary conditions

  connect(heatPort, heatTransfer.heatPorts[1]) annotation(
    Line(points = {{-100, 0}, {-74, 0}, {-74, 0}, {-74, 0}}, color = {191, 0, 0}));
  annotation(
    Documentation(info = "<html>
<p>
This base class extends PartialLumpedVolume with a vector of fluid ports and a replaceable wall HeatTransfer model.
</p>
<p>
The following modeling assumption are made:
<ul>
<li>homogeneous medium, i.e., phase separation is not taken into account,</li>
<li>no kinetic energy in the fluid, i.e., kinetic energy dissipates into the internal energy,</li>
<li>pressure loss definitions at vessel ports assume incompressible fluid,</li>
<li>outflow of ambient media is prevented at each port assuming check valve behavior.
If <code>fluidlevel &lt; portsData_height[i]</code> and <code>ports[i].p &lt; vessel_ps_static[i]</code> mass flow at the port is set to 0.</li>
</ul>
<p>
Each port has a (hydraulic) diameter and a height above the bottom of the vessel, which can be configured using the <b><code>portsData</code></b> record.
Alternatively the impact of port geometries can be neglected with <code>use_portsData=false</code>. This might be useful for early
design studies. Note that this means to assume an infinite port diameter at the bottom of the vessel.
Pressure drops and heights of the ports as well as kinetic and potential energy fluid entering or leaving the vessel are neglected then.
</p>
<p>
The following variables need to be defined by an extending model:
</p>
<ul>
<li><code>input fluidVolume</code>, the volume of the fluid in the vessel,</li>
<li><code>vessel_ps_static[nPorts]</code>, the static pressures inside the vessel at the height of the corresponding ports, at zero flow velocity, and</li>
<li><code>Wb_flow</code>, work term of the energy balance, e.g., p*der(V) if the volume is not constant or stirrer power.</li>
</ul>
<p>
An extending model should define:
</p>
<ul>
<li><code>parameter vesselArea</code> (default: Modelica.Constants.inf m2), the area of the vessel, to be related to cross flow areas of the ports for the consideration of dynamic pressure effects.</li>
</ul>
<p>
Optionally the fluid level may vary in the vessel, which effects the flow through the ports at configurable <code>portsData_height[nPorts]</code>.
This is why an extending model with varying fluid level needs to define:
</p>
<ul>
<li><code>input fluidLevel (default: 0m)</code>, the level the fluid in the vessel, and</li>
<li><code>parameter fluidLevel_max (default: 1m)</code>, the maximum level that must not be exceeded. Ports at or above fluidLevel_max can only receive inflow.</li>
</ul>
<p>
An extending model should not access the <code>portsData</code> record defined in the configuration dialog,
as an access to <code>portsData</code> may fail for <code>use_portsData=false</code> or <code>nPorts=0</code>.
</p>
<p>
Instead the predefined variables
</p>
<ul>
<li><code>portsData_diameter[nPorts]</code>,</li>
<li><code>portsData_height[nPorts]</code>,</li>
<li><code>portsData_zeta_in[nPorts]</code>, and</li>
<li><code>portsData_zeta_out[nPorts]</code></li>
</ul>
<p>
should be used if these values are needed.
</p>
</html>", revisions = "<html>
<ul>
<li><i>Jan. 2009</i> by R&uuml;diger Franke: extended with
<ul><li>portsData record and threat configurable port heights,</li>
 <li>consideration of kinetic and potential energy of fluid entering or leaving in energy balance</li>
</ul>
</li>
<li><i>Dec. 2008</i> by R&uuml;diger Franke: derived from OpenTank, in order to make general use of configurable port diameters</li>
</ul>
</html>"),
    Icon(coordinateSystem(initialScale = 0.1), graphics = {Text(lineColor = {0, 0, 255}, extent = {{-150, 110}, {150, 150}}, textString = "%name")}));
end PartialLumpedChamber_b;