within AICE.PowerUnits.Combustions.Chambers.BaseClasses.ReactionModels;

model PartialAngleBasedReaction2Zones
  extends PartialChamberReactionModel;

  constant Integer nReaction = 1;

  // cylinderBus interfaces
  Modelica.Blocks.Interfaces.RealInput phi_piston(final unit = "rad")
  annotation(
    Placement(
      visible = true,
      transformation(origin = {-70, 40}, extent = {{-5, -5}, {5, 5}}, rotation = -90)
    )
  );


  constant SI.StoichiometricNumber[nReaction, Medium.nXi]
    stoichiometric_reactants = fill(1/Medium.nXi, nReaction, Medium.nXi),
    stoichiometric_products  = fill(1/Medium.nXi, nReaction, Medium.nXi);

  parameter SI.Angle phi_reaction_begin "BTDC from piston compression TDC";
  parameter SI.AmountOfSubstance reactableAmount_eps = 1e-9;

  output SI.MolarFlowRate[n, nReaction] reactionRates;
  output SI.HeatFlowRate [n, nReaction] reactionHeatFlows;
  /////2zone用に追加/////////
  output SI.MolarFlowRate[Medium.nXi] MbXi_flow_u, MbXi_flow_b;
  output SI.HeatFlowRate [n, nReaction] reactionHeatFlows_u, reactionHeatFlows_b;
  /////////////////////////
  SI.AmountOfSubstance[n, nReaction]    reactableAmounts;

  // Trigger booleans
  Boolean[n, nReaction] reacts(each start = false);
  Boolean reaction_trigger = (
    Math.AngleDifference4Stroke.compare(phi_piston, 0 - phi_reaction_begin) >= 0
  );

  // Angle from current reaction begin
  discrete SI.Angle[n, nReaction] phi_currentReaction_begin;

  SI.Angle[n, nReaction] phi_currentReaction
    = fill(phi_piston, n, nReaction) - phi_currentReaction_begin[:,:];

  // Thermodynamic factors
  constant SI.MassFraction[Medium.nXi, Medium.nXi] mXis_identity = identity(Medium.nXi);

  SI.AbsolutePressure[n] ps = Medium.pressure(states);
  SI.Temperature[n]      Ts = Medium.temperature(states);

  SI.MolarInternalEnergy[n, Medium.nXi]
    Hs_Xi;

  SI.MolarInternalEnergy[n, nReaction]
    Hs_products,
    Hs_reactants,
    Hs_balance;

initial equation

  pre(reaction_trigger) = true;
  pre(reacts[:,:]) = fill(false, n, nReaction);

equation

  /**
   * Original output variables of `PartialChamberReactionModel`
   * - The molar reaction rate `MbXi_flow` of each substance
   * - The total reaction heat `Qb_flow`
   * are calculated from the reaction rate and heat for each reaction number.
   *
   * Complete model must implement `reaction*` variables showned below.
   * The variables are defined on each segment (1:n) and for each reaction NO (1:nReaction).
   * - `reactionRates[n, nReaction] (unit="mol/s")` reaction rate
   * - `reactionHeatFlows[n, Reaction] (unit="W")`  reaction heat
   */
  for iXi in 1:Medium.nXi loop
    //MbXi_flowは2zoneモデルでは使わないので計算不要だが，継承元PartialChamberReactionModelで変数が定義してあるので，計算式を残してある    
    MbXi_flow[iXi] = sum(
      (stoichiometric_products[jReaction, iXi] - stoichiometric_reactants[jReaction, iXi])
      * reactionRates[i, jReaction]
      for i in 1:n, jReaction in 1:nReaction
    );
    MbXi_flow_u[iXi] = sum(
      ( - stoichiometric_reactants[jReaction, iXi])
      * reactionRates[i, jReaction]
      for i in 1:n, jReaction in 1:nReaction
    );
    MbXi_flow_b[iXi] = sum(
      (stoichiometric_products[jReaction, iXi])
      * reactionRates[i, jReaction]
      for i in 1:n, jReaction in 1:nReaction
    );
  end for;

  Qb_flow = sum(reactionHeatFlows);

  /**
   * Remaining reactable amount of substance for `reactionRate` calcuration.
   * It is always 0 or more.
   */
  for i in 1:n loop
    for jReaction in 1:nReaction loop
      reactableAmounts[i, jReaction] = Functions.calcurateReactableAmount(
        stoichiometric = stoichiometric_reactants[jReaction, :],
        MXi = MXis[i, :]
      );
    end for;
  end for;

  for i in 1:n loop
    for jReaction in 1:nReaction loop

      /**
       * `reacts` :: reaction judgement
       * If {reactable amount is close to zero}              : stop     reaction (reacts = false)
       * Else if {at the moment trigger becomes true}        : start    reaction (reacts = true )
       * Else if {reaction has occured in the previous step} : continue reaction (reacts = true )
       * Else                                                : no       reaction (reacts = false)
       */
      reacts[i, jReaction] = (
        (reactableAmounts[i, jReaction] > reactableAmount_eps)
        and (edge(reaction_trigger) or pre(reacts[i, jReaction]))
      );

      /**
       * `phi_currentReaction` :: phi_piston at reaction begins (discrete)
       * If {at the moment reaction start} : update value
       * Else                              : hold   value
       */
      phi_currentReaction_begin[i, jReaction] = (
        if edge(reacts[i, jReaction])
        then phi_piston
        else pre(phi_currentReaction_begin[i, jReaction])
      );

    end for;
  end for;


  // # Thermodynamic factors
  // ## Molar specific enthalpies
  for i in 1:n loop
    for jXi in 1:Medium.nXi loop
      Hs_Xi[i, jXi] = Medium.specificEnthalpy_pTX(ps[i], Ts[i], mXis_identity[jXi,:]) * Medium.MMX[jXi];
    end for;

    for jReaction in 1:nReaction loop
      Hs_products [i, jReaction] = sum(Hs_Xi[i, :] .* stoichiometric_products [jReaction,:]);
      Hs_reactants[i, jReaction] = sum(Hs_Xi[i, :] .* stoichiometric_reactants[jReaction,:]);
    end for;
  end for;

  Hs_balance = Hs_reactants .- Hs_products;

end PartialAngleBasedReaction2Zones;