within AICE.PowerUnits.Combustions.Chambers.BaseClasses.ReactionModels;

model WiebeCombustion2Zones

  extends PartialAngleBasedReaction2Zones(
    final nReaction = Medium.nCombustion,
    final stoichiometric_products  = Medium.combustionStoichiometric_products,
    final stoichiometric_reactants = Medium.combustionStoichiometric_reactants
  );

  // cylinderBus interfaces
  Modelica.Blocks.Interfaces.BooleanOutput combusts
  annotation(
    Placement(
      visible = true,
      transformation(origin = {-50, 40}, extent = {{5, -5}, {-5, 5}}, rotation = -90)
    )
  );


  parameter SI.Angle phi_combustion_period "Anglar period until 1-exp(-a_wiebe) of fuel burns";
  parameter Real a_wiebe = -log(0.001);
  parameter Real m_wiebe "Wiebe constant of shape";

  /*
   * φ        :: [rad] piston angle from combustion begin
   * φ_period :: [rad] φ when almost (99.9%) fuel burned
   * θ        :: [1]   φ/φ_period,　dimentionlessed φ
   * n(θ)     :: [mol] fuel amount
   * besides, n(θ) ∝ reactable amount
   *
   * = = = =
   *
   * Gives the following approximation
   *
   *   dn               m
   *  ---- = - k * n * θ^
   *   dθ
   *
   * When integrated, the following equation is obtained:
   *
   *                             k       m+1
   * n(θ) = n(0) * exp ( -1 * ------- * θ^   )
   *                           m + 1
   *
   *                          ^^^^^^^
   *                        'a' constant
   *
   * 99.9% combusts ⇔ 0.001 remains
   *
   *               n(1)
   * ∴ a :: -ln( ------ ) = -ln(0.001) ~~ 6.9
   *               n(0)
   *
   * = = = =
   *
   *  dn           d(ln(n))
   * ---- =   n * ---------
   *  dt           dt
   *
   *               d(ln(n))     d(θ^m+1)
   *      =   n * ---------- * ----------
   *               d(θ^m+1)     dt
   *
   *                   d(θ^m+1)
   *      = - n * a * ----------
   *                   dt
   */

  Real[n, nReaction] theta     = phi_currentReaction[:,:] / phi_combustion_period;
  Real               der_theta = der(phi_piston)          / phi_combustion_period;

equation

  for i in 1:n loop
    for jReaction in 1:nReaction loop
      reactionRates[i, jReaction] = (
        if reacts[i, jReaction]
        then reactableAmounts[i,jReaction] * a_wiebe * (m_wiebe+1) * theta[i,jReaction] ^ m_wiebe * der_theta
        else 0.0
      );
    end for;

    reactionHeatFlows[i,:] = reactionRates[i,:] .* (Medium.combustionLHV[:] .+ Hs_balance[i,:]); //熱発生の計算に使うので2zoneでも残す
    //reactionHeatFlows[i,:] = reactionRates[i,:] .*  Hs_balance[i,:]; 
    
    /////2zone用に追加/////////
    reactionHeatFlows_u[i,:] = -reactionRates[i,:] .* Hs_reactants[i,:]; //unburned zone enthalpy loss
    reactionHeatFlows_b[i,:] = reactionRates[i,:] .* Hs_products[i,:]; // burned zone enthalpy gain
    //////////////////////////
    
  end for;

  // `combusts` :: Any reaction is occurring.
  combusts = Modelica.Math.BooleanVectors.anyTrue(
    Modelica.Math.BooleanVectors.anyTrue(
      reacts
    )
  );

end WiebeCombustion2Zones;