within AICE.PowerUnits.Combustions.Chambers;

model WiebeWoschniChamber
    "varying cylindric volume depending on the position of the piston"
  extends Icons.Fluid.SweptVolume;

  import Modelica.Constants.pi;

  parameter CarUnits.Volume V_displacement "Displacement volume of cylinder";
  parameter CarUnits.CompressionRatio compressionRatio "Compression ratio V_BDC/V_TDC";
  final parameter CarUnits.Volume V_TDC = V_displacement / (compressionRatio - 1);
  final parameter CarUnits.Volume V_BDC = V_TDC + V_displacement;

  parameter CarUnits.Diameter bore "Diameter of cylinder";
  final parameter CarUnits.Area pistonCrossArea = AICE.Math.diameter2area(bore)
    "Cross sectional area of piston";
  final parameter CarUnits.Length pistonStroke = V_displacement / pistonCrossArea;

  parameter SI.Angle phi_combustion_begin "BTDC from piston compression TDC";
  parameter SI.Angle phi_combustion_period "Anglar period until 1-exp(-a_wiebe) of fuel burns";

  parameter SI.AmountOfSubstance reactableAmount_eps = 1e-9;

  parameter Real a_wiebe = -log(0.001);
  parameter Real m_wiebe "Wiebe constant of shape";

  parameter Types.WoschniC1 c1_woschni_intakeExhaust    "Woschni C1 constant at intake, exhaust stroke";
  parameter Types.WoschniC1 c1_woschni_compressionPower "Woschni C1 constant at compression, power stroke";
  parameter Types.WoschniC2 c2_woschni                  "Woschni C2 constant";

  parameter Boolean use_phi_piston = false
  annotation(
    choices(checkBox = true)
  );
  parameter Boolean use_intakeValveOpens = false
  annotation(
    choices(checkBox = true)
  );
  parameter Boolean use_exhaustValveOpens = false
  annotation(
    choices(checkBox = true)
  );

  AICE.Blocks.Interfaces.AngleInput phi_piston
  annotation(
    Placement(
      visible = true,
      transformation(origin = {110, 80}, extent = {{10, -10}, {-10, 10}}, rotation = 0),
      iconTransformation(origin = {110, 80}, extent = {{10, -10}, {-10, 10}}, rotation = 0)
    )
  );

  Modelica.Blocks.Interfaces.BooleanInput intakeValveOpens
  annotation(
    Placement(
      visible = true,
      transformation(origin = {110, 60}, extent = {{10, -10}, {-10, 10}}, rotation = 0),
      iconTransformation(origin = {110, 60}, extent = {{10, -10}, {-10, 10}}, rotation = 0)
    )
  );

  Modelica.Blocks.Interfaces.BooleanInput exhaustValveOpens
  annotation(
    Placement(
      visible = true,
      transformation(origin = {110, 40}, extent = {{10, -10}, {-10, 10}}, rotation = 0),
      iconTransformation(origin = {110, 40}, extent = {{10, -10}, {-10, 10}}, rotation = 0)
    )
  );

  SI.Volume V "fluid volume";

  // Mass and energy balance, ports
  extends BaseClasses.PartialLumpedChamber(
    final fluidVolume = V,
    redeclare final model ReactionModel = BaseClasses.ReactionModels.WiebeCombustion,
    redeclare final model HeatTransfer  = BaseClasses.HeatTransfers.WoschniHeatTransfer,
    heatTransfer(
      surfaceAreas={
        pistonCrossArea * 2 // piston head + chamber ceil
        + 2*sqrt(pistonCrossArea*pi)*(flange.s+V_TDC/pistonCrossArea)
      },
      bore = bore,
      pistonStroke = pistonStroke,
      V_BDC = V_BDC,
      c1_intakeExhaust    = c1_woschni_intakeExhaust,
      c1_compressionPower = c1_woschni_compressionPower,
      c2                  = c2_woschni
    ),
    reactionModel(
      phi_reaction_begin = phi_combustion_begin,
      phi_combustion_period = phi_combustion_period,
      reactableAmount_eps = reactableAmount_eps,
      a_wiebe = a_wiebe,
      m_wiebe = m_wiebe
    )
  );

  Modelica.Mechanics.Translational.Interfaces.Flange_b flange
    "translation flange for piston" annotation (Placement(transformation(
          extent={{-10,90},{10,110}})));

equation
  assert(V >= 0, "Chamber volume must not be smaller than zero!");

  // volume size
  V = V_TDC + flange.s * pistonCrossArea;

  0 = flange.f + (medium.p - system.p_ambient) * pistonCrossArea;

  // energy balances
  Wb_flow = medium.p * pistonCrossArea * (-der(flange.s));

  // definition of port pressures
  for i in 1:nPorts loop
    vessel_ps_static[i] = medium.p;
  end for;

  // Connections of input => output variables
  connect(phi_piston, heatTransfer .phi_piston);
  connect(phi_piston, reactionModel.phi_piston);
  connect(intakeValveOpens , heatTransfer.intakeValveOpens );
  connect(exhaustValveOpens, heatTransfer.exhaustValveOpens);
  connect(reactionModel.combusts, heatTransfer.combusts);

end WiebeWoschniChamber;
