within AICE.PowerUnits.Combustions.Chambers;

model WiebeWoschniChamber2zones "varying cylindric volume depending on the position of the piston"
  extends Icons.Fluid.SweptVolume;
  import Modelica.Constants.pi;
  package Medium = Media.Interfaces.PartialCombustionMixtureMedium "Medium in the reaction model";
  parameter CarUnits.Volume V_displacement "Displacement volume of cylinder";
  parameter CarUnits.CompressionRatio compressionRatio "Compression ratio V_BDC/V_TDC";
  final parameter CarUnits.Volume V_TDC = V_displacement / (compressionRatio - 1);
  final parameter CarUnits.Volume V_BDC = V_TDC + V_displacement;
  parameter CarUnits.Diameter bore "Diameter of cylinder";
  final parameter CarUnits.Area pistonCrossArea = AICE.Math.diameter2area(bore) "Cross sectional area of piston";
  final parameter CarUnits.Length pistonStroke = V_displacement / pistonCrossArea;
  parameter SI.Angle phi_combustion_begin "BTDC from piston compression TDC";
  parameter SI.Angle phi_combustion_period "Anglar period until 1-exp(-a_wiebe) of fuel burns";
  parameter SI.AmountOfSubstance reactableAmount_eps = 1e-9;
  parameter Real a_wiebe = -log(0.001);
  parameter Real m_wiebe "Wiebe constant of shape";
  parameter Types.WoschniC1 c1_woschni_intakeExhaust "Woschni C1 constant at intake, exhaust stroke";
  parameter Types.WoschniC1 c1_woschni_compressionPower "Woschni C1 constant at compression, power stroke";
  parameter Types.WoschniC2 c2_woschni "Woschni C2 constant";
  parameter Boolean use_phi_piston = false annotation(
    choices(checkBox = true));
  parameter Boolean use_intakeValveOpens = false annotation(
    choices(checkBox = true));
  parameter Boolean use_exhaustValveOpens = false annotation(
    choices(checkBox = true));
  parameter CarUnits.AirByFuelRatio airByFuel = 15.0;
  parameter SI.MassFraction[Medium.nX] X_mixture = (airByFuel * Medium.reference_X[:] .+ 1 * Medium.reference_X_fuel[:]) / (airByFuel + 1);
  parameter Real V_ratio_u = 0.999;
  parameter Real V_ratio_b = 1 - V_ratio_u;
  Boolean intakeValveCloses = not intakeValveOpens;
  Integer state_2volumes;

  AICE.Blocks.Interfaces.AngleInput phi_piston annotation(
    Placement(visible = true, transformation(origin = {110, 80}, extent = {{10, -10}, {-10, 10}}, rotation = 0), iconTransformation(origin = {110, 80}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
  Modelica.Blocks.Interfaces.BooleanInput intakeValveOpens annotation(
    Placement(visible = true, transformation(origin = {110, 60}, extent = {{10, -10}, {-10, 10}}, rotation = 0), iconTransformation(origin = {110, 60}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
  Modelica.Blocks.Interfaces.BooleanInput exhaustValveOpens annotation(
    Placement(visible = true, transformation(origin = {110, 40}, extent = {{10, -10}, {-10, 10}}, rotation = 0), iconTransformation(origin = {110, 40}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
  SI.Volume V "fluid volume";
  SI.Volume Vu "volume of unburned zone";
  SI.Volume Vb(start = V_TDC * 0.01) "volume of burned zone";
  
  extends BaseClasses.PartialLumpedChamber_u(
    final fluidVolume = Vu, 
    redeclare final model ReactionModel = BaseClasses.ReactionModels.WiebeCombustion2Zones, 
    redeclare final model HeatTransfer = BaseClasses.HeatTransfers.WoschniHeatTransfer, 
      X_start = X_mixture, 
      heatTransfer(surfaceAreas = {(pistonCrossArea * 2 + 
                                  2 * sqrt(pistonCrossArea * pi) * (flange.s + V_TDC / pistonCrossArea)) * (Vu / V)}, 
      bore = bore, 
      pistonStroke = pistonStroke, 
      V_BDC = V_BDC,
      c1_intakeExhaust = c1_woschni_intakeExhaust, 
      c1_compressionPower = c1_woschni_compressionPower, 
      c2 = c2_woschni
      ), 
                                  
    reactionModel(phi_reaction_begin = phi_combustion_begin, phi_combustion_period = phi_combustion_period, reactableAmount_eps = reactableAmount_eps, a_wiebe = a_wiebe, m_wiebe = m_wiebe));

  replaceable BaseClasses.PartialLumpedChamber_b chamberZone_b(
    use_HeatTransfer = true,
    p_start = 32.06e+5, 
    T_start = 1262, 
    final fluidVolume = Vb, 
    mb_flow_react = mb_flow_react, 
    mbXi_flow_p = mbXi_flow_p, 
    Qb_comb = reactionModel.Qb_flow, 
    redeclare replaceable package Medium = Medium, 
    redeclare final model HeatTransfer = BaseClasses.HeatTransfers.WoschniHeatTransfer, 
    heatTransfer(redeclare package Medium = Medium, 
      surfaceAreas = {(pistonCrossArea * 2 + 2 * sqrt(pistonCrossArea * pi) * (flange.s + V_TDC / pistonCrossArea)) * (Vb / V)}, 
      bore = bore, 
      pistonStroke = pistonStroke, 
      V_BDC = V_BDC, 
      c1_intakeExhaust = c1_woschni_intakeExhaust, 
      c1_compressionPower = c1_woschni_compressionPower, 
      c2 = c2_woschni), 
      X_start = X_mixture
      
  ) annotation(
    Placement(visible = true, transformation(origin = {-2, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));

  Modelica.Mechanics.Translational.Interfaces.Flange_b flange "translation flange for piston" annotation(
    Placement(transformation(extent = {{-10, 90}, {10, 110}})));

initial equation
  state_2volumes = 1;

equation
  assert(V >= 0, "Chamber volume must not be smaller than zero!");

  V = Vu + Vb;
  state_2volumes = pre(state_2volumes);

  when edge(intakeValveCloses) then
    reinit(state_2volumes, 1);
//IVC時（燃焼前）は成分の移動はやらない．EVO時から圧縮に応じて決まった容積は状態量に燃焼に応じた化学種とエネルギーが逐次輸入されればよい．
/*reinit(Vb, V_ratio_b * V);
    reinit(Vu, V_ratio_u * V);
    
    reinit(chamberZone_b.medium.h, pre(medium.h));
    reinit(medium.h, pre(medium.h));
    reinit(chamberZone_b.medium.p, pre(medium.p));
    reinit(medium.p, pre(medium.p));
    reinit(chamberZone_b.mXi, pre(mXi)*V_ratio_b);
    reinit(mXi, pre(mXi)*V_ratio_u);
    reinit(chamberZone_b.m, pre(m)*V_ratio_b);
    reinit(m, pre(m)*V_ratio_u); 
    */
  end when;
  when edge(exhaustValveOpens) then
    reinit(state_2volumes, 0);
    reinit(Vb, V_ratio_b * V);
    reinit(Vu, V_ratio_u * V);
// Burn側からunburn側へ輸送
    reinit(medium.p, pre(medium.p));
    reinit(medium.h, (pre(medium.h) * pre(m) + pre(chamberZone_b.medium.h) * pre(chamberZone_b.m) * (1 - V_ratio_b)) / (pre(m) + pre(chamberZone_b.m) * (1 - V_ratio_b)));
    reinit(medium.T, Medium.temperature_phX(medium.p, (pre(medium.h) * pre(m) + pre(chamberZone_b.medium.h) * pre(chamberZone_b.m) * (1 - V_ratio_b)) / (pre(m) + pre(chamberZone_b.m) * (1 - V_ratio_b)), (pre(medium.Xi) * pre(m) + pre(chamberZone_b.medium.Xi) * pre(chamberZone_b.m) * (1 - V_ratio_b)) / (pre(m) + pre(chamberZone_b.m) * (1 - V_ratio_b))));
    for i in 1:Medium.nXi loop
      reinit(medium.Xi[i], (mXi[i] + chamberZone_b.mXi[i] * (1 - V_ratio_b)) / (pre(m) + pre(chamberZone_b.m) * (1 - V_ratio_b)));
    end for;
//Burn側はVolumeが変われば圧力，温度，比エンタルピは変わらなくて良いので以下のreinitはあってもなくても同じ
/*
    reinit(chamberZone_b.medium.p, pre(chamberZone_b.medium.p));
    reinit(chamberZone_b.medium.h, pre(chamberZone_b.medium.h));
    reinit(chamberZone_b.medium.T, pre(chamberZone_b.medium.T));
    for i in 1:Medium.nXi loop
      reinit(chamberZone_b.medium.Xi[i], pre(chamberZone_b.medium.Xi[i]));
    end for;
    */
  end when;
//zone pressure balance and zone expansion work
  chamberZone_b.medium.p = max(100, min(1E8, medium.p));
  Wb_flow = medium.p * (-der(Vu));
  chamberZone_b.Wb_flow = chamberZone_b.medium.p * der(-Vb);

  if reactionModel.combusts then
    Hb_flow = sum(ports_H_flow) + sum(ports_E_flow) + sum(reactionModel.reactionHeatFlows_u[:, :]);
// apply reaction enthalpy consumption
    chamberZone_b.Hb_flow = sum(reactionModel.reactionHeatFlows_b[:, :]);
  else
    Hb_flow = sum(ports_H_flow) + sum(ports_E_flow);
    chamberZone_b.Hb_flow = 0.0;
  end if;

  chamberZone_b.heatTransfer.phi_piston = phi_piston;
  chamberZone_b.heatTransfer.combusts = reactionModel.combusts;
  chamberZone_b.heatTransfer.intakeValveOpens = intakeValveOpens;
  chamberZone_b.heatTransfer.exhaustValveOpens = exhaustValveOpens;
// volume size
  V = V_TDC + flange.s * pistonCrossArea;

  0 = flange.f + (medium.p - system.p_ambient) * pistonCrossArea;
// definition of port pressures
  for i in 1:nPorts loop
    vessel_ps_static[i] = medium.p;
  end for;
// Connections of input => output variables
  connect(phi_piston, heatTransfer.phi_piston);
  connect(phi_piston, reactionModel.phi_piston);
  connect(intakeValveOpens, heatTransfer.intakeValveOpens);
  connect(exhaustValveOpens, heatTransfer.exhaustValveOpens);
  connect(reactionModel.combusts, heatTransfer.combusts);
  connect(chamberZone_b.heatPort, heatPort);
   annotation(
    experiment(StartTime = 0, StopTime = 0.001, Tolerance = 1e-6, Interval = 2e-6),
    __OpenModelica_simulationFlags(lv = "LOG_STATS", outputFormat = "mat", s = "dassl"));
end WiebeWoschniChamber2zones;