
# `MultipleDiscreteGear` 検討事項

以下の記述は、`MultipleDiscreteGear` の実装にあたって、検討すべきと予想される技術的事項についてまとめる。実装方法によっては、下記の検討事項が問題にならない可能性がある。構築者の判断で採用すること。

## `flange_a.phi`, `flange_b.phi` のC0級 連続の担保

MSLのコードを見ていると、ギア比が一定であることを前提に、ギア比 := 回転角度の比としている。

```modelica
// CLASS: Modelica.Mechanics.Rotational.Components.IdealGear
    phi_a = ratio*phi_b;

// CLASS: Modelica.Mechanics.Rotational.Components.LossyGear
    phi_a = ratio*phi_b;
```

しかし、このような回転角度の比を拘束する実装ではギア比が変わったとき、phi_bが不自然な動きをする。

| `t` | `phi_a` | `phi_b` | `phi_a`:`phi_b` |
| --- | ------- | ------- | --------------- |
| 1 | 10 | 20 | 1:2 |
| 2 | 20 | 40 | 1:2 |
| 3 | 30 | 60 | 1:2 |
| 4 | 40 | 80 | 1:2 |
| 5 | 50 | 100 | 1:2 |
| 6 | 60 | __60__ | __1:1__ |
| 7 | 70 | __70__ | __1:1__ |
| 8 | 80 | __80__ | __1:1__ |
| 9 | 90 | __90__ | __1:1__ |

本来は以下のようになるべきである。

| `t` | `phi_a` | `phi_b` | `phi_a`:`phi_b` |
| --- | ------- | ------- | --------------- |
| 1 | 10 | 20 | 1:2 |
| 2 | 20 | 40 | 1:2 |
| 3 | 30 | 60 | 1:2 |
| 4 | 40 | 80 | 1:2 |
| 5 | 50 | 100 | 1:2 |
| 6 | 60 | 110 | __1:1__ |
| 7 | 70 | 120 | __1:1__ |
| 8 | 80 | 130 | __1:1__ |
| 9 | 90 | 140 | __1:1__ |

この表から分かる通り、ギア比とは回転角度の比ではなく、__回転速度の比__ である。
期待される結果の表に回転速度を書き込むと、以下の表のようになる。

| `t` | `omega_a` | `omega_b` | `omega_a`:`omega_b` | `phi_a` | `phi_b` |
| --- | --------- | --------- | ------------------- | ------- | ------- |
| 1 | 10 | 20 | 1:2 | 10 | 20 |
| 2 | 10 | 20 | 1:2 | 20 | 40 |
| 3 | 10 | 20 | 1:2 | 30 | 60 |
| 4 | 10 | 20 | 1:2 | 40 | 80 |
| 5 | 10 | 20 | 1:2 | 50 | 100 |
| 6 | 10 | 10 | 1:1 | 60 | 110 |
| 7 | 10 | 10 | 1:1 | 70 | 120 |
| 8 | 10 | 10 | 1:1 | 80 | 130 |
| 9 | 10 | 10 | 1:1 | 90 | 140 |

ギア比が一定でない場合、たとえば以下のように実装したほうがいいと考えられる。

```diff
++    Modelica.SIunits.AngularVelocity w_a = der(phi_a);
++    Modelica.SIunits.AngularVelocity w_b = der(phi_b);
equation
--    phi_a = ratio*phi_b;
++    w_a = ratio*w_b
```

## `discrete Real` 変数の適切な使用

Modelicaは、常微分方程式を解く関係で、変数が不変であるか可変であるか、可変ならば連続であるか連続でないかを区別している。この判定はコンパイラが式から判定をするので、多くの場合は気にしなくてもよい。
今回のモデルでは、`gearPosition`によって`ratios_gear[:]`や`etas_gear[:]`から選択されるギア比、効率は、実数であるが不連続となる。こういったコードを書くと、しばしばコンパイラが変数の連続性判定に失敗し、コンパイルが失敗することがある。

このような事態を避けるために、明らかに不連続となる実数変数については、独立させて`discrete Real`変数とすることで、コンパイラに不連続な変数であることを伝えることができる。

```modelica
  // Modelica.SIunits.* のクラスは、Realの言い換えであるので、discrete Realと同じ意味になる。
  discrete Modelica.SIunits.DimensionlessRatio ratio = ratios_gear[gearPosition];
  discrete Modelica.SIunits.Efficiency eta = etas_gear[gearPosition];
```

## ニュートラルの表現

ニュートラルのとき、どういう動作をするか考える。

- 負荷がかからない、動力が伝達されない <=> `flange_a.tau = 0`, `flange_b.tau = 0`
- `flange_a`, `flange_b` は自由に回転する <=> `flange_a.phi`, `flange_b.phi` に拘束条件がない

つまり、ギアが入っているときは回転数`der(phi)`を`ratios_gear`で拘束するが、ニュートラル時は`tau`を拘束するように切り替わるということがわかる。

ひとつの変数について拘束する式が切り替わるのではなく、(複数の)拘束する変数を切り替えるようなモデルを書くことが求められる。

以下は拘束する変数を切り替えたモデルが動作することを確認するサンプルコードである。

```modelica
model SwitchVar

  Real x, x_plus1;

  parameter Modelica.SIunits.Time period = 0.25;

equation

  x_plus1 = x + 1;

  /**
   * 拘束する変数を `x` <=> `x_plus1` と入れ替える
   * period周期で入れ替わり、前半は`x`を、後半は`x_plus1`を拘束する。
   */
  if mod(time, period) < (period / 2.0) then
    x       = time;
  else
    x_plus1 = time;
  end if;

end SwitchVar;
```

[仕様書に戻る](MultipleDiscreteGear.md)
